/**********************************************************************************
 * $URL: https://source.sakaiproject.org/svn/course-management/tags/sakai_2-2-3/cm-api/api/src/java/org/sakaiproject/coursemanagement/api/EnrollmentSet.java $
 * $Id: EnrollmentSet.java 9151 2006-05-09 05:40:34Z jholtzman@berkeley.edu $
 **********************************************************************************
 *
 * Copyright (c) 2005 The Regents of the University of California
 * 
 * Licensed under the Educational Community License Version 1.0 (the "License");
 * By obtaining, using and/or copying this Original Work, you agree that you have read,
 * understand, and will comply with the terms and conditions of the Educational Community License.
 * You may obtain a copy of the License at:
 * 
 *      https://source.sakaiproject.org/svn/sakai/trunk/sakai_license_1_0.html
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/
package org.sakaiproject.coursemanagement.api;

import java.util.Set;

/**
 * Defines a group of students who are somehow associated with a CourseOffering
 * or a Section for credit.  Defines who is allowed to submit the final grade for this
 * student, and what the grade is for.
 * 
 * @author <a href="mailto:jholtzman@berkeley.edu">Josh Holtzman</a>
 */
public interface EnrollmentSet {

	/**
	 * A unique enterprise id
	 * @return
	 */
	public String getEid();

	/**
	 * The title
	 * @return
	 */
	public String getTitle();

	/**
	 * A description
	 * @return
	 */
	public String getDescription();

	/**
	 * A category
	 * @return
	 */
	public String getCategory();
	
	/**
	 * The default credits an Enrollment should have, if not specified by the
	 * Enrollment itself.
	 * @return
	 */
	public String getDefaultEnrollmentCredits();
	
	/**
	 * The official grader(s) for this EnrollmentSet.
	 * @return
	 */
	public Set getOfficialInstructors();
}
