/**********************************************************************************
*
* $Id: GradebookConfiguration.java 10817 2006-06-16 20:52:56Z lance@indiana.edu $
*
***********************************************************************************
*
* Copyright (c) 2006 The Regents of the University of California, The MIT Corporation
*
* Licensed under the Educational Community License, Version 1.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*      http://www.opensource.org/licenses/ecl1.php
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*
**********************************************************************************/
package org.sakaiproject.service.gradebook.shared;

import java.util.*;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.sakaiproject.service.gradebook.shared.GradebookService;

/**
 * Simple configuration bean. Set up the properties you want changed and
 * call init() when the GradebookService is wired up. (We can't set them
 * directly by injecting the GradebookService bean because that would
 * bypass transaction proxying.)
 */
public class GradebookConfiguration {
    private static final Log log = LogFactory.getLog(GradebookConfiguration.class);

    private Collection gradingScaleDefinitions;
    private String defaultGradingScale;
    private GradebookService gradebookService;

	public void init() {
		if (gradebookService != null) {
			if (gradingScaleDefinitions != null) {
				gradebookService.setAvailableGradingScales(gradingScaleDefinitions);
			}
			if (defaultGradingScale != null) {
				gradebookService.setDefaultGradingScale(defaultGradingScale);
			}
		}
	}

	public void setAvailableGradingScales(Collection gradingScaleDefinitions) {
		this.gradingScaleDefinitions = gradingScaleDefinitions;
	}

	public void setDefaultGradingScale(String defaultGradingScale) {
		this.defaultGradingScale = defaultGradingScale;
	}

	public void setGradebookService(GradebookService gradebookService) {
		this.gradebookService = gradebookService;
	}
}
